CraftBarData = {};

-- Bindings
BINDING_HEADER_CRAFTBARHEADER = "CraftBar";
BINDING_NAME_CRAFTBARSNAP = "MouseSnap";

local MASQUE = LibStub("Masque", true);
local buttongroup = nil;

local initialized = false;

local craftbar_professions = {
	['01Tailoring'] = {spellid=3908, recipes=true},
	['02Enchanting'] = {spellid=7411, recipes=true},
	['03Disenchant'] = {spellid=13262, recipes=false},
	['04Smelting'] = {spellid=2656, recipes=true},
	['05Blacksmithing'] = {spellid=3100, recipes=true},
	['06Alchemy'] = {spellid=2259, recipes=true},
	['07Leatherworking'] = {spellid=2108, recipes=true},
	['08Engineering'] = {spellid=4036, recipes=true},
	['09Jewelcrafting'] = {spellid=25229, recipes=true},
	['10Inscription'] = {spellid=45357, recipes=true},
	['11Runeforging'] = {spellid=53428, recipes=true},
	['12Survey'] = {spellid=77364, recipes=false},
	['13Archaeology'] = {spellid=78670, recipes=false},
	['14Cooking'] = {spellid=2550, recipes=true},
	['15Basic Campfire'] = {spellid=818, recipes=false},
	['16First Aid'] = {spellid=3273, recipes=true},
}

local craftbar_buttons = {
	['01Tailoring'] = nil,
	['02Enchanting'] = nil,
	['03Disenchant'] = nil,
	['04Smelting'] = nil,
	['05Blacksmithing'] = nil,
	['06Alchemy'] = nil,
	['07Leatherworking'] = nil,
	['08Engineering'] = nil,
	['09Jewelcrafting'] = nil,
	['10Inscription'] = nil,
	['11Runeforging'] = nil,
	['12Survey'] = nil,
	['13Archaeology'] = nil,
	['14Basic Campfire'] = nil,
	['15First Aid'] = nil,
}

function craftbar_debug(value, name)
    if (name) then
 	    DEFAULT_CHAT_FRAME:AddMessage("debug "..name..": "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    else
 	    DEFAULT_CHAT_FRAME:AddMessage("debug: "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    end
end

-- functions
function CraftBar_OnLoad(self)

 	DEFAULT_CHAT_FRAME:AddMessage("|c333399ffNirriti's |rCraftBar loaded.", 1.0, 1.0, 1.0, 1, 10);

 	self:RegisterEvent("ADDON_LOADED");

	self:RegisterEvent("PLAYER_ENTER_COMBAT");
	self:RegisterEvent("PLAYER_LEAVE_COMBAT");
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	
	self:RegisterEvent("TRADE_SKILL_SHOW");
	
	--self:RegisterAllEvents();

	SLASH_CRAFTBAR1 = "/craftbar";
  	SLASH_CRAFTBAR2 = "/craft";
  	SlashCmdList["CRAFTBAR"] = function(msg)
		CraftBar_SlashCommandHandler(msg);
	end
end

function CraftBar_OnEvent(self, event, ...)

    --craftbar_debug(event, "event");
    
	if (event == "ADDON_LOADED") then
	
		local addonName = ...;
		if (addonName == "CraftBar") then	
		    if (MASQUE) then
                buttongroup = MASQUE:Group("CraftBar");
            end
		end
	end
	
	if (event == "PLAYER_ENTER_COMBAT" or event == "PLAYER_REGEN_DISABLED") then
	    local frame = getglobal("CraftBarFrame");
	    frame:Hide();
	end
end

function CraftBar_SlashCommandHandler(msg)
	if (msg == "reload") then
	 	ReloadUI();
	end
end

function CraftBar_OnUpdate(self, elapsed)
    CraftBar_UpdateCooldowns(self);
end

function CraftBar_UpdateCooldowns(self)
		
	local cooldown = _G[self:GetName().."Cooldown"];
	
    local item = self:GetAttribute("item");
    local spell = self:GetAttribute("spell");
    
    local start, duration, enable = GetSpellCooldown(spell);
  	
    if (cooldown and start and duration and enable) then
        CooldownFrame_SetTimer(cooldown, start, duration, enable);
    end
end

function CraftBar_initButtons()
 	
    --[[CraftBarFrame:SetBackdrop( { 
        --bgFile = "Interface/DialogFrame/UI-DialogBox-Background", 
        edgeFile = "Interface/Tooltips/UI-Tooltip-Border", tile = true, tileSize = 16, edgeSize = 16, 
        insets = { left = 4, right = 4, top = 4, bottom = 4 }
    });]]
    
    CraftBarFrame:Hide();

	for name, data in pairs(craftbar_professions) do 	 
	    local spellid = data.spellid;
 	    local spellname, rank, texture, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spellid);
 	
 	    -- Button in der Buttonleiste
        local button1 = CreateFrame("Button", name, CraftBarFrame, "CraftBarButtonTemplate");
        
        button1:RegisterForClicks("AnyUp");
        button1.rangeTimer = -1;
	
        button1:RegisterEvent("PLAYER_TARGET_CHANGED");
        button1:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
        button1:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");

    	button1:SetAttribute("type", "spell");
		button1:SetAttribute("spell", spellname);
		button1:SetAttribute("id", spellid);
		
		if (MASQUE) then
		    buttongroup:AddButton(button1);
		end
			
		_G[button1:GetName().."Icon"]:SetTexture(texture); 
		
		craftbar_buttons[name] = {bar=button1};
 	end
 	
 	initialized = true;
end

function CraftBar_initCraftFrame()	  

    if (not initialized) then    
        CraftBar_initButtons();	
    end 

    local count = 0;

	for name, data in pairs(craftbar_professions) do 	 
	    local spellid = data.spellid;	    
 	    local spellname, rank, texture, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spellid);
 	    local usable, nomana = IsUsableSpell(spellname);
 	    
        local button = craftbar_buttons[name].bar;

 	    if (usable == 1) then 	    
            button:ClearAllPoints();
            button:SetPoint("LEFT", (6 + 38 * count), 0);
			
            _G[button:GetName().."Icon"]:SetTexture(GetSpellTexture(spellname));
 	    
 	        button:Show();
 	        count = count + 1;
 	    else
 	        button:Hide();
 	    end
 	end
 	
    CraftBarFrame:SetWidth(10 + 38 * count);
    
    return count;
end  

function CraftBar_SnapDown()

	if (InCombatLockdown()) then
	    UIErrorsFrame:AddMessage(ERR_NOT_IN_COMBAT, 1.0, 0.0, 0.0, 1, 10);
	end	

	local frame = getglobal("CraftBarFrame");
	
	local count = CraftBar_initCraftFrame();
	
	if (count < 1) then
	    frame:Hide();
	    return;
	end
	
	def_posx = frame:GetLeft();
	def_posy = frame:GetBottom();
	
	local x, y = GetCursorPosition();
	local scale = UIParent:GetEffectiveScale();
	
	x = x / scale;	
	y = y / scale;
	
	frame:ClearAllPoints();
	frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);
	
	frame:Show();
end

function CraftBar_SnapUp()
	local frame = getglobal("CraftBarFrame");
	frame:ClearAllPoints();
	frame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", def_posx, def_posy);
	
    if (not InCombatLockdown()) then
	    frame:Hide();
	end
end

function CraftBar_MouseOver(button)
	CraftBar_SetTooltip(button);
end

function CraftBar_MouseOut(button)
	GameTooltip:FadeOut();
	selectedButton = nil;
end

function CraftBar_SetTooltip(button)

    local spellid = button:GetAttribute("id");
    
    GameTooltip_SetDefaultAnchor(GameTooltip, button);
    
    GameTooltip:SetSpellByID(spellid);
        
    GameTooltip:AppendText("");
end